// file      : CCF/IDL2/SemanticGraph/Sequence.hpp
// author    : Boris Kolpackov <boris@dre.vanderbilt.edu>
// cvs-id    : Sequence.hpp,v 1.3 2005/10/26 16:45:18 boris Exp

#ifndef CCF_IDL2_SEMANTIC_GRAPH_SEQUENCE_HPP
#define CCF_IDL2_SEMANTIC_GRAPH_SEQUENCE_HPP

#include "CCF/IDL2/SemanticGraph/Elements.hpp"
#include "CCF/IDL2/SemanticGraph/IntExpression.hpp"

namespace CCF
{
  namespace IDL2
  {
    namespace SemanticGraph
    {
      //
      //
      //
      class Sequence : public virtual Specialization
      {
      public:
        Type&
        type () const
        {
          return
            dynamic_cast<ArgumentsWithType&> (**arguments_begin ()).type ();
        }

        virtual bool
        complete () const
        {
          return type ().complete ();
        }

        static Introspection::TypeInfo const&
        static_type_info ();

      protected:
        friend class Graph<Node, Edge>;

        Sequence ()
        {
          type_info (static_type_info ());
        }
      };


      //
      //
      //
      class UnboundedSequence : public virtual Sequence
      {
      public:
        static Introspection::TypeInfo const&
        static_type_info ();

      protected:
        friend class Graph<Node, Edge>;

        UnboundedSequence ()
        {
          type_info (static_type_info ());
        }
      };


      //
      //
      //
      class BoundedSequence : public virtual Sequence
      {
      public:
        IntExpression&
        bound () const
        {
          ArgumentsIterator i (arguments_begin ());

          ++i; // Bound is always second to the type.

          return
            dynamic_cast<IntExpression&> (
              dynamic_cast<ArgumentsWithValue&> (**i).value ());
        }

        static Introspection::TypeInfo const&
        static_type_info ();

      protected:
        friend class Graph<Node, Edge>;

        BoundedSequence ()
        {
          type_info (static_type_info ());
        }
      };
    }
  }
}

#endif  // CCF_IDL2_SEMANTIC_GRAPH_SEQUENCE_HPP
